import QtQuick 2.0
import QtGraphicalEffects 1.0

MouseArea {
    property QtObject popoverModel: model
    anchors.fill: parent
    onClicked: popoverModel.reject()

    Item { 
        id: container

        width:  rect.width  + (2 * rectShadow.radius);
        height: rect.height + (2 * rectShadow.radius);

        x: { 
            if (popoverModel.elementRect.x + width/2 > root.width) { 
                root.width - popoverModel.elementRect.x - units.gu(5) 
            } else if (popoverModel.elementRect.x - width/2 + popoverListView.contentItem.width/2 < 0) { 
                units.gu(3.75) 
            } else { 
                popoverModel.elementRect.x - width/2 + units.gu(6.25)
            }
        }

        y: {  
            if (popoverModel.elementRect.y + popoverModel.elementRect.height + height < parent.height ) {
                popoverDownCaret.visible = true  
                popoverUpCaret.visible = false  
                popoverModel.elementRect.y + popoverModel.elementRect.height
            } else { 
                popoverDownCaret.visible = false  
                popoverUpCaret.visible = true  
                popoverModel.elementRect.y - height 
            }
        }

        Rectangle {
            id: rect 
            width: units.gu(36)
            height: units.gu(37.5) //( popoverListView.contentItem.height < 300 ) ? popoverListView.contentItem.height + 40 : 300
            radius: units.gu(0.625) 
            anchors.centerIn: parent
            antialiasing: true;
            color: "gray"

            Text {
                id: popoverUpCaret 
                anchors { left: parent.horizontalCenter; margins: - units.gu(1.25); top: parent.bottom; topMargin: - units.gu(2.75); }
                text: "\uF0D7"
                color: "gray"
                font { family: fontAwesome.name; pointSize: units.gu(5) } 
            }
            Text {
                id: popoverDownCaret 
                anchors { left: parent.horizontalCenter; margins: - units.gu(1.25); top: parent.top; topMargin: - units.gu(4); }
                text: "\uF0D8"
                color: "gray"
                font { family: fontAwesome.name; pointSize: units.gu(5) } 
            }

            ListView {
                id: popoverListView
                anchors { fill: parent; margins: units.gu(2.5); topMargin: units.gu(5); bottomMargin: units.gu(5) }
                model: popoverModel.items

                delegate: Rectangle {
                    color: "transparent"
                    height: units.gu(5)
                    width: parent.width

                    Text {
                        anchors { left: parent.left; leftMargin: units.gu(1.25); right: parent.right; rightMargin: units.gu(1.25); }
                        text: model.text
                        color: "white"
                        font { pointSize: units.gu(1.8); weight: Font.Bold }
                        elide: Text.ElideRight
                    }

                    Text { // highlight 
                        visible: model.selected ? true : false 
                        color: "lightgray"; text: "\uF00C"; 
                        anchors.right : parent.right  
                        font { family: fontAwesome.name; pointSize: units.gu(2.3) }
                    }

                    MouseArea {
                        anchors.fill: parent
                        enabled: model.enabled
                        onClicked: { popoverModel.accept(model.index); }
                    }
                }
            }
        }
    }
    DropShadow {
        id: rectShadow;                
        anchors.fill: source
        cached: true;                          
        horizontalOffset: units.gu(0.38);
        verticalOffset: units.gu(0.38);
        radius: units.gu(1.5);
        samples: units.gu(2);
        color: "#80000000";
        smooth: true;
        source: container;
    }
}
